/*	Copyright (C) 2005, 2006 by Andy Grebe a.k.a. tecknophreak(tecknophreak@gmail.com)
 *
 *	This file is part of TouchMe Games.
 *
 *	TouchMe Games is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	TouchMe Games is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TouchMe Games; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DartCricket10_H
#define DartCricket10_H

#include <iostream>
#include <string>
#include <map>
#include <gameptr.h>
#include <termios.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dartlib.h>

#define dartCricketPath "/usr/local/share/touchmegames/dart/"

class DartCricket10 : public GAME_PTR {
private:
	int round;
	GtkWidget *fixed;
	GtkWidget *roundLabel;
	GtkWidget *ppdWindow;
	GtkWidget *ppdVbox;
	GtkWidget *ppdPlayerLabel;
	GtkWidget *ppdLabel;
	GtkWidget *lowTon;
	int lowTonY;
	GtkWidget *highTon;
	int highTonY;
	GtkWidget *ton80;
	int ton80Y;
	bool didTon;
	GtkRcStyle *menuUp, *menuDown;
	
	GtkWidget *popupMenu;
	
	std::string gameName;
	GtkWidget *tophbox;
	GtkWidget *largeScore;
	GtkWidget *smallScore;
	GtkWidget *userScores[8];
	GtkWidget *markImages[8][7];
	GtkWidget *centerBg;
	GtkWidget *currentLabel;
	gint timer;
	int totalScore;
	bool doubleBullOn, lastChanceDarts;
	int dartGameType; // 101, 301, 501, 701, 901
	int numberOfPlayers;
	int currentPlayer;
	int currentScore;
	int currentDart;
	int gameState;
	enum GAME_STATES {
		PLAYING,
		BETWEEN_ROUNDS,
		DROPPING_LOW_TON,
		DROPPING_HIGH_TON,
		DROPPING_TON_80,
		GAME_OVER1,
		GAME_OVER2
	};
	
	// Serial Port settings
	DartLib dartBoard;
	
	// Statistics
	int darts[8];
	int finalScore[8];
	int rounds[8];
	int doubles[8];
	int triples[8];
	int bulls[8];
	int lowTons[8];
	int highTons[8];
	int ton80s[8];
	int maxTon[8];
	char closeMarks[8][7];
	
	int lastDartScore[5];
	int lastDartMult[5];
	
	int winner;
	int beginningScore; // for when the player busts
	double highScore;
	
	// interface
	// opening screen
	GtkWidget *table;
	GtkWidget *gameTypeEntry, *numPlayersEntry, *doubleBullLabel, *lastChanceLabel;
	
	// playing screen
	GtkWidget *lhvbox, *rhvbox, *midvbox;
	
	GtkWidget *createOpenScreen();
	void createPPDWindow();
	void createGameOver1();
	void createLowTon();
	void createHighTon();
	void createTon80();
	void createPopupMenu();
	
	void openSerial();
	
	// game screen
	void initScreen();
	
	GtkWidget *playerLabels[8];
	GtkWidget *playerScoreLabels[8];
	GtkWidget *currentTotalLabel;
	GtkWidget *currentScoreLabel;
	GtkWidget *dartImages[3];
	GtkWidget *dartLabels[3];
	GtkWidget *playerLabel;
	// events
	static void exitGame(GtkWidget *widget, GdkEvent *event, gpointer ptr);
	static void showMenu(GtkWidget *widget, GdkEvent *event, gpointer ptr);
	static void changeNumPlayers(GtkWidget *widget, gpointer ptr);
	static void changeGame(GtkWidget *widget, gpointer ptr);
	static void onOffToggle(GtkWidget *widget, gpointer ptr);
	static gboolean timeout(gpointer ptr);
	
	static void startGame(GtkWidget *widget, gpointer ptr);
	
	void runGameOver();
	
	// menu events
	static gboolean eventPressed(GtkWidget *widget, GdkEventKey *event, gpointer ptr);
	static gboolean missedDartReleased(GtkWidget *widget, GdkEventKey *event, gpointer ptr);
	static gboolean removeLastDartReleased(GtkWidget *widget, GdkEventKey *event, gpointer ptr);
	static gboolean exitGameReleased(GtkWidget *widget, GdkEventKey *event, gpointer ptr);
	static gboolean cancelReleased(GtkWidget *widget, GdkEventKey *event, gpointer ptr);
	
	// score is the value, not the total score * mult
	void addDart(int score, int mult);
	void removeDart();
	void resyncGame();
	
	SCORE_STRUCT scores[10];
	GdkColor bgcolor;
	
	// support.cpp
	int getWinner();
	int getMarks(int player);
public:
	DartCricket10();
	virtual ~DartCricket10();
	
	std::string name() const;
	std::string imageName() const;
	GdkColor *backgroundColor();
	// initializes the game's variables and returns the
	// top level h/vbox
	GtkWidget *init();
	GtkWidget *getHelp();
	int gameType() const;
	unsigned int score() const { return 0; };
};

#endif // DartCricket10_H
